//-----------------------------------------------------------------------------
// EXT_LCD_EA_eDIP_TFT
// Version 1.0
// 27.11.2012 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Fuer ein externes LC-Display
// Typ          : EA-eDIP-Display
// Modell       : z.B. EA-eDIP_TFT-32
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_EXT_MODULE/"
// liegen und mit #include <UB_EXT_MODULE/EXT_LCD_EA_eDIP_TFT.h>
// eingebunden werden
//          
//----------------------------------------------------------------------------- 
// Belegt : Port-B (Bit4)
// Benutzt : MK3_2560_SPI.h = SPI = Port-B (Bit1 bis Bit3)
//-----------------------------------------------------------------------------
//
// Patchplan : (fuer den Mode : SPI)
//
// (in der Default-Einstellung liegt alles auf Port-B)
//
// Port-B1 (SPI-SCK)  muss zum Display Pin CLK
// Port-B2 (SPI-MOSI) muss zum Display Pin MOSI
// Port-B3 (SPI-MISO) muss zum Display Pin MISO
// Port-B4 (CS) muss zum Display Pin SS
//
// Hinweis :
//
// Die Hardware vom Display muss so eingestellt sein :
//
//  Pin-5  : RESET   = Hi-Pegel -> Reset inaktiv
//  Pin-11 : SPIMODE = Lo-Pegel -> Datenuebertragung per SPI 
//  Pin-10 : DORD    = Lo-Pegel -> MSB-First
//  Pin-14 : CPOL    = Lo-Pegel -> CPOL=0
//  Pin-15 : CPHA    = Lo-Pegel -> CPHA=0
//
//-----------------------------------------------------------------------------



//----------------------------------------------------------------------------
// Includes
//---------------------------------------------------------------------------- 

#include <stdio.h>
#include "ub_mk3_2560/MK3_2560_SPI.h" //  bindet die SPI-Funktionen ein


//-----------------------------------------------------------------------------
// Defines
//-----------------------------------------------------------------------------
#define  EA_EDIP_PUFFER_SIZE   50   // groesse vom Daten-Puffer (in Bytes)

#define  EA_EDIP_PAUSE_10US    10  // ca. 10us Pause (fuer das EA-Display)
#define  EA_EDIP_PAUSE_100US   100 // ca. 100us Pause (fuer das EA-Display)

#define  EA_EDIP_ANZ_VERSUCHE  10  // Versuche um Display zu finden



//-----------------------------------------------------------------------------
// Port-Pin definition
//-----------------------------------------------------------------------------

#define  EA_EDIP_DDR    DDRB
#define  EA_EDIP_PORT   PORTB
#define  EA_EDIP_CS     0x10    // Bit4


//-----------------------------------------------------------------------------
// Display Groesse
// (Bei EA-eDIP_TFT-32 -> Pixel_X=320, Pixel_Y=240)
//----------------------------------------------------------------------------
#define  EA_EDIP_PIXEL_X  320   // Anzahl der Pixel in X-Richtung
#define  EA_EDIP_PIXEL_Y  240   // Anzahl der Pixel in Y-Richtung
   




//-----------------------------------------------------------------------------
// Funktionen die vom MAIN benutzt werden koennen
//----------------------------------------------------------------------------- 
void EXT_LCD_INIT_EA_EDIP(void);           // init vom LC-Display
bool LCD_EA_EDIP_CHECK(void);              // test ob Display vorhanden
int LCD_EA_EDIP_READ_TOUCH(void);          // Touch auslesen
void LCD_EA_EDIP_CMD_01(unsigned char nr); // Beispiel Kommando 01 
void LCD_EA_EDIP_CMD_02(void);             // Beispiel Kommando 02




//-----------------------------------------------------------------------------
// Display Befehle
//-----------------------------------------------------------------------------
#define  EA_EDIP_DC1     0x11  // DC1 Byte vom EA-Display
#define  EA_EDIP_DC2     0x12  // DC2 Byte vom EA-Display
#define  EA_EDIP_ACK     0x06  // ACK Byte vom EA-Display
#define  EA_EDIP_NACK    0x15  // NACK Byte vom EA-Display
#define  EA_EDIP_ESC     0x1B  // Escape Char vom EA-Display

#define  EA_EDIP_CHR_A   0x41  // Char 'A'
#define  EA_EDIP_CHR_B   0x42  // Char 'B'
#define  EA_EDIP_CHR_C   0x43  // Char 'C'
#define  EA_EDIP_CHR_D   0x44  // Char 'D'
#define  EA_EDIP_CHR_E   0x45  // Char 'E'
#define  EA_EDIP_CHR_F   0x46  // Char 'F'
#define  EA_EDIP_CHR_G   0x47  // Char 'G'
#define  EA_EDIP_CHR_H   0x48  // Char 'H'
#define  EA_EDIP_CHR_I   0x49  // Char 'I'
#define  EA_EDIP_CHR_J   0x4A  // Char 'J'
#define  EA_EDIP_CHR_K   0x4B  // Char 'K'
#define  EA_EDIP_CHR_L   0x4C  // Char 'L'
#define  EA_EDIP_CHR_M   0x4D  // Char 'M' 
#define  EA_EDIP_CHR_N   0x4E  // Char 'N'
#define  EA_EDIP_CHR_O   0x4F  // Char 'O'
#define  EA_EDIP_CHR_P   0x50  // Char 'P'
#define  EA_EDIP_CHR_Q   0x51  // Char 'Q'
#define  EA_EDIP_CHR_R   0x52  // Char 'R'
#define  EA_EDIP_CHR_S   0x53  // Char 'S'
#define  EA_EDIP_CHR_T   0x54  // Char 'T'
#define  EA_EDIP_CHR_U   0x55  // Char 'U'
#define  EA_EDIP_CHR_V   0x56  // Char 'V'
#define  EA_EDIP_CHR_W   0x57  // Char 'W'
#define  EA_EDIP_CHR_X   0x58  // Char 'X'
#define  EA_EDIP_CHR_Y   0x59  // Char 'Y'
#define  EA_EDIP_CHR_Z   0x5A  // Char 'Z'
#define  EA_EDIP_DUMMY   0xFF  // Dummy Byte zum lesen von Daten



//-----------------------------------------------------------------------------
// interne Variabeln
//-----------------------------------------------------------------------------
unsigned char EA_EDIP_PUFFER[EA_EDIP_PUFFER_SIZE];




//-----------------------------------------------------------------------------
// interne Funktionen
//-----------------------------------------------------------------------------
int EA_EDIP_check_puffer(void);
int EA_EDIP_read_puffer(unsigned char laenge);
int EA_EDIP_check_Data(void);
int EA_EDIP_send_dc(unsigned char typ, unsigned char laenge);
unsigned char EA_EDIP_send(unsigned char wert);



//-----------------------------------------------------------------------------
// Init vom LC-Display
//-----------------------------------------------------------------------------
void EXT_LCD_INIT_EA_EDIP(void) { // init vom LC-Display
  // Display-Pins auf Ausgang
  EA_EDIP_DDR|=EA_EDIP_CS;

  // Display-Pins auf Hi-Pegel
  EA_EDIP_PORT|=EA_EDIP_CS;

  // Init vom SPI-Bus
  MK3_INIT_SPI();

  // SPI-Einstellungen
  // CPOL=0  CPHA=0
  MK3_SPI_SET_MODE(SPI_MODE_0); 

  //  Teiler=128
  MK3_SPI_SET_FRQ(SPI_CLK_DIV_128);

  // Display testen
  LCD_EA_EDIP_CHECK();
}


//-----------------------------------------------------------------------------
// test ob Display vorhanden ist
// ret_wert = true, wenn Display vorhanden
//-----------------------------------------------------------------------------
bool LCD_EA_EDIP_CHECK(void) {               // test ob Display vorhanden
  bool ret_wert=false;
  int check;
  unsigned char n;

  for(n=0;n<EA_EDIP_ANZ_VERSUCHE;n++) {
    check=EA_EDIP_check_puffer();
    if(check>=0) {
      ret_wert=true;
      break;
    }
  }

  if(check>0) {
    // Puffer auslesen (Daten verwerfen)
    n=check;
    EA_EDIP_read_puffer(n);
  }

  return(ret_wert);
}


//-----------------------------------------------------------------------------
// Touch vom Display auslesen
// Rueckgabe vom Touchcode (wenn eine Taste gedrueckt bzw. losgelassen wurde)
// 
// ret_wert : 0  = keine Touch-Daten vorhanden
// ret_wert : >0 = Touch-Code der Taste (1...255)
// ret_wert : <0 = Fehler
//-----------------------------------------------------------------------------
int LCD_EA_EDIP_READ_TOUCH(void) { // Touch auslesen
  int ret_wert=-1;
  int check;
  unsigned char laenge;

  check=EA_EDIP_check_puffer();
  if(check>0) {
    laenge=check;
    check=EA_EDIP_read_puffer(laenge);
    if(check==0) {
      check=EA_EDIP_check_Data();
      if(check>0) {
        // Touch-Code zurueckgeben
        ret_wert=check;
      }
    }   
  }
  else if(check==0) {
    // keine Daten vorhanden
    ret_wert=0;
  }

  return(ret_wert);
}


//-----------------------------------------------------------------------------
// Display-Kommando starten (dient als Beispielcode)
// Gruppe   : Befehle fuer Makros
// Funktion : normales Makro starten
// -> normales Makro mit der Nummer "nr" wird gestartet
//-----------------------------------------------------------------------------
void LCD_EA_EDIP_CMD_01(unsigned char nr) { // Beispiel Kommando 01

  // Kommando zusammenbauen
  EA_EDIP_PUFFER[0]=EA_EDIP_ESC;   // ESC
  EA_EDIP_PUFFER[1]=EA_EDIP_CHR_M; // Makro
  EA_EDIP_PUFFER[2]=EA_EDIP_CHR_N; // normales Makro
  EA_EDIP_PUFFER[3]=nr;            // Nummer

  // Kommando senden
  // "DC1,4,ESC,M,N,nr,CRC"
  EA_EDIP_send_dc(EA_EDIP_DC1,4); // Laenge = 4    
}


//-----------------------------------------------------------------------------
// Display-Kommando starten (dient als Beispielcode)
// Gruppe   : Grafik Funktionen
// Funktion : Display loeschen
// -> Displayinhalt loeschen (mit Hintergrundfarbe fuellen)
//-----------------------------------------------------------------------------
void LCD_EA_EDIP_CMD_02(void) { // Beispiel Kommando 02

  // Kommando zusammenbauen
  EA_EDIP_PUFFER[0]=EA_EDIP_ESC;   // ESC
  EA_EDIP_PUFFER[1]=EA_EDIP_CHR_D; // Display
  EA_EDIP_PUFFER[2]=EA_EDIP_CHR_L; // loeschen

  // Kommando senden
  // "DC1,3,ESC,D,L,CRC"
  EA_EDIP_send_dc(EA_EDIP_DC1,3); // Laenge = 3    
}



//-----------------------------------------------------------------------------
// Check ob was im Display-Puffer steht
//-----------------------------------------------------------------------------
int EA_EDIP_check_puffer(void) { // interne Funktion
  int ret_wert=-1;
  int check,n;
  unsigned char crc;

  // Kommando zusammenbauen  
  EA_EDIP_PUFFER[0]=EA_EDIP_CHR_I; // Puffer Info anfordern

  // Kommando senden
  // "DC2,1,I,CRC"
  check=EA_EDIP_send_dc(EA_EDIP_DC2,1); // Laenge = 1 

  if(check==0) {
    // wenn ACK empfangen
    // 4 Bytes abholen
    crc=0;
    for(n=0;n<=3;n++) {
      EA_EDIP_PUFFER[n]=EA_EDIP_send(EA_EDIP_DUMMY);
      crc+=EA_EDIP_PUFFER[n];
    }

    // CRC abholen
    EA_EDIP_PUFFER[4]=EA_EDIP_send(EA_EDIP_DUMMY);
    
    // check der empfangenen Daten
    ret_wert=0;
    if(EA_EDIP_PUFFER[0]!=EA_EDIP_DC2) ret_wert=-3; // Fehler beim empfang
    if(EA_EDIP_PUFFER[1]!=2) ret_wert=-4;           // Fehler beim empfang
    if(EA_EDIP_PUFFER[4]!=crc) ret_wert=-5;         // Fehler bei der CRC

    // Anzahl der Bytes im Puffer auslesen
    if(ret_wert==0) {      
      // Anzahl zurueckgeben
      ret_wert=EA_EDIP_PUFFER[2];
    }
  }
  else {
    // kein ACK empfangen
    ret_wert=-2;
    if(check==-1) ret_wert=-1; // NACK empfangen
    if(check==-2) ret_wert=-2; // was anderes empfangen
  }

  return(ret_wert);
}


//-----------------------------------------------------------------------------
// Display-Puffer auslesen
//-----------------------------------------------------------------------------
int EA_EDIP_read_puffer(unsigned char laenge) { // interne Funktion
  int ret_wert=-1;
  int check;
  unsigned char crc;
  unsigned int n;

  if(laenge<(EA_EDIP_PUFFER_SIZE-3)) {

    // Kommando zusammenbauen  
    EA_EDIP_PUFFER[0]=EA_EDIP_CHR_S; // Puffer Daten anfordern

    // Kommando senden
    // "DC2,1,S,CRC"
    check=EA_EDIP_send_dc(EA_EDIP_DC2,1); // Laenge = 1

    if(check==0) {
      // wenn alles OK

      // n Bytes Daten abholen (plus Header)
      crc=0;
      for(n=0;n<(laenge+2);n++) {
        EA_EDIP_PUFFER[n]=EA_EDIP_send(EA_EDIP_DUMMY);
        crc+=EA_EDIP_PUFFER[n];
      }

      // CRC abholen
      EA_EDIP_PUFFER[laenge+2]=EA_EDIP_send(EA_EDIP_DUMMY);
    
      // check der empfangenen Daten
      ret_wert=0;
      if(EA_EDIP_PUFFER[0]!=EA_EDIP_DC1) ret_wert=-3; // Fehler beim empfang
      if(EA_EDIP_PUFFER[laenge+2]!=crc) ret_wert=-4;  // Fehler bei der CRC
    }
    else {
      // kein ACK empfangen
      ret_wert=-2;
    }
  }
  else {
    // puffer reicht nicht
    ret_wert=-1;
  }
  
  return(ret_wert);
}


//-----------------------------------------------------------------------------
// Display-Puffer auswerten
//-----------------------------------------------------------------------------
int EA_EDIP_check_Data(void) { // interne Funktion
  int ret_wert=-1;
  unsigned char laenge;

  laenge=EA_EDIP_PUFFER[1]; // laenge der Daten
  if(laenge==4) {
    // ein 4 Byte Kommando auswerten
    ret_wert=0;
    if(EA_EDIP_PUFFER[2]!=EA_EDIP_ESC) ret_wert=-2;   // Fehler beim empfang
    if(EA_EDIP_PUFFER[3]!=EA_EDIP_CHR_A) ret_wert=-3; // Fehler beim empfang

    // Touch-Code der Taste auslesen
    if(ret_wert==0) {      
      ret_wert=EA_EDIP_PUFFER[5];
    }
  }
  else {
    ret_wert=-1;
  }

  return(ret_wert);
}

//-----------------------------------------------------------------------------
// DC-Kommando an das Display senden
//-----------------------------------------------------------------------------
int EA_EDIP_send_dc(unsigned char typ, unsigned char laenge) { // interne Funktion
  int ret_wert=-2;
  unsigned char n,crc,wert;
  unsigned int p;

  // startkennung senden
  EA_EDIP_send(typ);
  crc=typ;

  // laenge senden
  EA_EDIP_send(laenge);
  crc+=laenge;

  // kommando senden
  for(n=0;n<laenge;n++) {
    EA_EDIP_send(EA_EDIP_PUFFER[n]);
    crc+=EA_EDIP_PUFFER[n];
  }

  // checksumme senden
  EA_EDIP_send(crc);

  // 10us warten (WICHTIG !!)
  for(p=0;p<EA_EDIP_PAUSE_10US;p++) {
    // nur warten
  }

  // Dummy-Byte senden
  // damit ACK oder NACK empfangen werden kann
  wert=EA_EDIP_send(EA_EDIP_DUMMY);

  // auswertung vom Return-Wert
  if(wert==EA_EDIP_ACK) {
    // wenn ACK empfangen
    ret_wert=0;
  }
  else if(wert==EA_EDIP_NACK) {
    // wenn NACK empfangen
    ret_wert=-1;
  }
  else {
    // wenn etwas anderes empfangen
    ret_wert=-2;
  }

  return(ret_wert);
}


//-----------------------------------------------------------------------------
// ein Byte senden und empfangen
//-----------------------------------------------------------------------------
unsigned char EA_EDIP_send(unsigned char wert) { // interne Funktion
  unsigned char ret_wert=0;
  unsigned int p;

  // Chip-Select auf Lo
  EA_EDIP_PORT&=~EA_EDIP_CS;

  //---------------------------------------------------
  //---- Byte per SPI senden (und lesen)
  //---------------------------------------------------
  ret_wert=MK3_SPI_SEND(wert);

  // Chip-Select auf Hi      
  EA_EDIP_PORT|=EA_EDIP_CS;

  // 100us warten (WICHTIG !!)
  for(p=0;p<EA_EDIP_PAUSE_100US;p++) {
    // nur warten
  }

  return(ret_wert);
}